@@ -1,4 +1,4 @@
-/* $Id: log.c,v 1.15 2001/10/25 17:27:39 rjkaes Exp $
+/* $Id: log.c,v 1.22 2002/06/15 17:37:11 rjkaes Exp $
  *
  * Logs the various messages which tinyproxy produces to either a log file or
  * the syslog daemon. Not much to it...
@@ -19,7 +19,10 @@
 
 #include "tinyproxy.h"
 
+#include "hashmap.h"
+#include "heap.h"
 #include "log.h"
+#include "utils.h"
 
 static char *syslog_level[] = {
 	NULL,
@@ -39,12 +42,21 @@ static char *syslog_level[] = {
 /*
  * Store the log level setting.
  */
-static short int log_level = LOG_ERR;
+static int log_level = LOG_INFO;
+
+/*
+ * Hold a listing of log messages which need to be sent once the log
+ * file has been established.
+ * The key is the actual messages (already filled in full), and the value
+ * is the log level.
+ */
+static hashmap_t log_message_storage;
 
 /*
  * Set the log level for writing to the log file.
  */
-void set_log_level(short int level)
+void
+set_log_level(int level)
 {
 	log_level = level;
 }
@@ -52,11 +64,11 @@ void set_log_level(short int level)
 /*
  * This routine logs messages to either the log file or the syslog function.
  */
-void log_message(short int level, char *fmt, ...)
+void
+log_message(int level, char *fmt, ...)
 {
 	va_list args;
 	time_t nowtime;
-	FILE *cf;
 
 	char time_string[TIME_LENGTH];
 #if defined(HAVE_SYSLOG_H) && !defined(HAVE_VSYSLOG_H)
@@ -74,17 +86,40 @@ void log_message(short int level, char *fmt, ...)
 		if (level > LOG_INFO && level != LOG_CONN)
 			return;
 	} else if (level > log_level)
-			return;
+		return;
 #endif
-	
 
 #ifdef HAVE_SYSLOG_H
-	if (config.syslog && level == LOG_CONN) 
+	if (config.syslog && level == LOG_CONN)
 		level = LOG_INFO;
 #endif
 
 	va_start(args, fmt);
 
+	/*
+	 * If the config file hasn't been processed, then we need to store
+	 * the messages for later processing.
+	 */
+	if (!processed_config_file) {
+		char string_level[4];
+
+		if (!log_message_storage) {
+			log_message_storage = hashmap_create(1);
+			if (!log_message_storage)
+				return;
+		}
+
+		vsnprintf(str, STRING_LENGTH, fmt, args);
+		snprintf(string_level, 4, "%d", level);
+
+		hashmap_insert(log_message_storage, str,
+			       string_level, strlen(string_level) + 1);
+
+		va_end(args);
+
+		return;
+	}
+
 #ifdef HAVE_SYSLOG_H
 	if (config.syslog) {
 #  ifdef HAVE_VSYSLOG_H
@@ -95,22 +130,65 @@ void log_message(short int level, char *fmt, ...)
 #  endif
 	} else {
 #endif
+		FILE* log_file;
+		int fd;
+
+		fd = create_file_safely(config.logf_name, FALSE);
+		log_file = fdopen(fd, "w+");
+
 		nowtime = time(NULL);
 		/* Format is month day hour:minute:second (24 time) */
 		strftime(time_string, TIME_LENGTH, "%b %d %H:%M:%S",
 			 localtime(&nowtime));
 
-		if (!(cf = config.logf))
-			cf = stderr;
+		fprintf(log_file, "%-9s %s [%ld]: ", syslog_level[level],
+			time_string, (long int) getpid());
+		vfprintf(log_file, fmt, args);
+		fprintf(log_file, "\n");
 
-		fprintf(cf, "%-9s %s [%ld]: ", syslog_level[level],
-			time_string, (long int)getpid());
-		vfprintf(cf, fmt, args);
-		fprintf(cf, "\n");
-		fflush(cf);
+		fclose(log_file);
 #ifdef HAVE_SYSLOG_H
 	}
 #endif
 
 	va_end(args);
 }
+
+/*
+ * This needs to send any stored log messages.
+ */
+void
+send_stored_logs(void)
+{
+	hashmap_iter iter;
+	char *level_string;
+	char *string;
+	int level;
+
+	iter = hashmap_first(log_message_storage);
+	if (iter >= 0) {
+		for ( ; !hashmap_is_end(log_message_storage, iter); ++iter) {
+			hashmap_return_entry(log_message_storage,
+					     iter,
+					     &string,
+					     (void **)&level_string);
+
+			level = atoi(level_string);
+
+#if NDEBUG
+			if (log_level == LOG_CONN && level == LOG_INFO)
+				continue;
+			else if (log_level == LOG_INFO) {
+				if (level > LOG_INFO && level != LOG_CONN)
+					continue;
+			} else if (level > log_level)
+				continue;
+#endif
+
+			log_message(level, string);
+		}
+	}
+
+	hashmap_delete(log_message_storage);
+	log_message_storage = NULL;
+}
