@@ -1,4 +1,4 @@
-/* $Id: sock.c,v 1.15 2001/11/12 21:10:51 rjkaes Exp $
+/* $Id: sock.c,v 1.38 2002/05/31 18:08:01 rjkaes Exp $
  *
  * Sockets are created and destroyed here. When a new connection comes in from
  * a client, we need to copy the socket and the create a second socket to the
@@ -23,20 +23,31 @@
 
 #include "tinyproxy.h"
 
-#include "dnscache.h"
 #include "log.h"
+#include "heap.h"
 #include "sock.h"
-#include "utils.h"
+#include "text.h"
 
 /*
- * The mutex is used for locking around any calls which access global
- * variables.
- *	- rjkaes
+ * Take a string host address and return a struct in_addr so we can connect
+ * to the remote host.
+ *
+ * Return a negative if there is a problem.
  */
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+static int
+lookup_domain(struct in_addr *addr, const char *domain)
+{	
+	struct hostent* result;
 
-#define LOCK()   pthread_mutex_lock(&mutex);
-#define UNLOCK() pthread_mutex_unlock(&mutex);
+	assert(domain != NULL);
+
+	result = gethostbyname(domain);
+	if (result) {
+		memcpy(addr, result->h_addr_list[0], result->h_length);
+		return 0;
+	} else
+		return -1;
+}
 
 /* This routine is so old I can't even remember writing it.  But I do
  * remember that it was an .h file because I didn't know putting code in a
@@ -49,36 +60,57 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
  * dotted-decimal form before it does a name lookup.
  *      - rjkaes
  */
-int opensock(char *ip_addr, uint16_t port)
+int
+opensock(char *ip_addr, uint16_t port)
 {
 	int sock_fd;
 	struct sockaddr_in port_info;
+	struct sockaddr_in bind_addr;
 	int ret;
 
 	assert(ip_addr != NULL);
 	assert(port > 0);
 
-	memset((struct sockaddr*)&port_info, 0, sizeof(port_info));
+	memset((struct sockaddr *) &port_info, 0, sizeof(port_info));
 
 	port_info.sin_family = AF_INET;
 
 	/* Lookup and return the address if possible */
-	ret = dnscache(&port_info.sin_addr, ip_addr);
+	ret = lookup_domain(&port_info.sin_addr, ip_addr);
 
 	if (ret < 0) {
-		log_message(LOG_ERR, "opensock: Could not lookup address \"%s\".", ip_addr);
+		log_message(LOG_ERR,
+			    "opensock: Could not lookup address \"%s\".",
+			    ip_addr);
 		return -1;
 	}
 
 	port_info.sin_port = htons(port);
 
 	if ((sock_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-		log_message(LOG_ERR, "opensock: socket() error \"%s\".", strerror(errno));
+		log_message(LOG_ERR, "opensock: socket() error \"%s\".",
+			    strerror(errno));
 		return -1;
 	}
 
-	if (connect(sock_fd, (struct sockaddr*)&port_info, sizeof(port_info)) < 0) {
-		log_message(LOG_ERR, "opensock: connect() error \"%s\".", strerror(errno));
+	/* Bind to the specified address */
+	if (config.bind_address) {
+		memset(&bind_addr, 0, sizeof(bind_addr));
+		bind_addr.sin_family = AF_INET;
+		bind_addr.sin_addr.s_addr = inet_addr(config.bind_address);
+
+		ret = bind(sock_fd, (struct sockaddr *)&bind_addr, sizeof(bind_addr));
+		if (ret < 0) {
+			log_message(LOG_ERR, "Could not bind local address \"%\" because of %s",
+				    config.bind_address,
+				    strerror(errno));
+			return -1;
+		}
+	}
+
+	if (connect(sock_fd, (struct sockaddr *) &port_info, sizeof(port_info)) < 0) {
+		log_message(LOG_ERR, "opensock: connect() error \"%s\".",
+			    strerror(errno));
 		return -1;
 	}
 
@@ -88,7 +120,8 @@ int opensock(char *ip_addr, uint16_t port)
 /*
  * Set the socket to non blocking -rjkaes
  */
-int socket_nonblocking(int sock)
+int
+socket_nonblocking(int sock)
 {
 	int flags;
 
@@ -101,7 +134,8 @@ int socket_nonblocking(int sock)
 /*
  * Set the socket to blocking -rjkaes
  */
-int socket_blocking(int sock)
+int
+socket_blocking(int sock)
 {
 	int flags;
 
@@ -117,7 +151,8 @@ int socket_blocking(int sock)
  * the pointer, while the socket is returned as a default return.
  *	- rjkaes
  */
-int listen_sock(uint16_t port, socklen_t *addrlen)
+int
+listen_sock(uint16_t port, socklen_t * addrlen)
 {
 	int listenfd;
 	const int on = 1;
@@ -138,10 +173,18 @@ int listen_sock(uint16_t port, socklen_t *addrlen)
 	} else {
 		addr.sin_addr.s_addr = inet_addr("0.0.0.0");
 	}
-	
-	bind(listenfd, (struct sockaddr *)&addr, sizeof(addr));
 
-	listen(listenfd, MAXLISTEN);
+	if (bind(listenfd, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
+		log_message(LOG_ERR, "Unable to bind listening socket because of %s",
+			    strerror(errno));
+		return -1;
+	}
+
+	if (listen(listenfd, MAXLISTEN) < 0) {
+		log_message(LOG_ERR, "Unable to start listening socket because of %s",
+			    strerror(errno));
+		return -1;
+	}
 
 	*addrlen = sizeof(addr);
 
@@ -149,138 +192,40 @@ int listen_sock(uint16_t port, socklen_t *addrlen)
 }
 
 /*
- * Takes a socket descriptor and returns the string contain the peer's
- * IP address.
+ * Return the peer's socket information.
  */
-char *getpeer_ip(int fd, char *ipaddr)
+int
+getpeer_information(int fd, char* ipaddr, char* string_addr)
 {
 	struct sockaddr_in name;
 	size_t namelen = sizeof(name);
+	struct hostent* result;
 
 	assert(fd >= 0);
 	assert(ipaddr != NULL);
+	assert(string_addr != NULL);
 
 	/*
-	 * Make sure the user's buffer is initialized to an empty string.
+	 * Clear the memory.
 	 */
-	*ipaddr = '\0';
+	memset(ipaddr, '\0', PEER_IP_LENGTH);
+	memset(string_addr, '\0', PEER_STRING_LENGTH);
 
-	if (getpeername(fd, (struct sockaddr*)&name, &namelen) != 0) {
-		log_message(LOG_ERR, "getpeer_ip: getpeername() error \"%s\".", strerror(errno));
+	if (getpeername(fd, (struct sockaddr *)&name, &namelen) != 0) {
+		log_message(LOG_ERR, "getpeer_information: getpeername() error: %s",
+			    strerror(errno));
+		return -1;
 	} else {
 		strlcpy(ipaddr,
-			inet_ntoa(*(struct in_addr*)&name.sin_addr.s_addr),
+			inet_ntoa(*(struct in_addr *)&name.sin_addr.s_addr),
 			PEER_IP_LENGTH);
 	}
 
-	return ipaddr;
-}
-
-/*
- * Takes a socket descriptor and returns the string containing the peer's
- * address.
- */
-char *getpeer_string(int fd, char *string)
-{
-	struct sockaddr_in name;
-	size_t namelen = sizeof(name);
-	struct hostent *peername;
-
-	assert(fd >= 0);
-	assert(string != NULL);
-
-	/*
-	 * Make sure the user's buffer is initialized to an empty string.
-	 */
-	*string = '\0';
-
-	if (getpeername(fd, (struct sockaddr *)&name, &namelen) != 0) {
-		log_message(LOG_ERR, "getpeer_string: getpeername() error \"%s\".", strerror(errno));
+	result = gethostbyaddr((char *)&name.sin_addr.s_addr, 4, AF_INET);
+	if (result) {
+		strlcpy(string_addr, result->h_name, PEER_STRING_LENGTH);
+		return 0;
 	} else {
-		LOCK();
-		peername = gethostbyaddr((char *)&name.sin_addr.s_addr,
-					 sizeof(name.sin_addr.s_addr),
-					 AF_INET);
-		if (peername)
-			strlcpy(string, peername->h_name, PEER_STRING_LENGTH);
-		else
-			log_message(LOG_ERR, "getpeer_string: gethostbyaddr() error \"%s\".", hstrerror(h_errno));
-
-		UNLOCK();
+		return -1;
 	}
-
-	return string;
-}
-
-/*
- * Write the buffer to the socket. If an EINTR occurs, pick up and try
- * again.
- */
-ssize_t safe_write(int fd, const void *buffer, size_t count)
-{
-	ssize_t len;
-	
-	do {
-		len = write(fd, buffer, count);
-	} while (len < 0 && errno == EINTR);
-
-	return len;
-}
-
-/*
- * Matched pair for safe_write(). If an EINTR occurs, pick up and try
- * again.
- */
-ssize_t safe_read(int fd, void *buffer, size_t count)
-{
-	ssize_t len;
-
-	do {
-		len = read(fd, buffer, count);
-	} while (len < 0 && errno == EINTR);
-
-	return len;
-}
-
-/*
- * Reads in a line of text one character at a time. Finishes when either a
- * newline is detected, or maxlen characters have been read. The function
- * will actually copy one less than maxlen into the buffer. In other words,
- * the returned string will _always_ be '\0' terminated.
- */
-ssize_t readline(int fd, char *ptr, size_t maxlen)
-{
-	size_t n;
-	ssize_t rc;
-	char c;
-
-	assert(fd >= 0);
-	assert(ptr != NULL);
-
-	for (n = 1; n < maxlen; n++) {
-	again:
-		if ((rc = recv(fd, &c, 1, MSG_NOSIGNAL)) == 1) {
-			*ptr++ = c;
-			if (c == '\n')
-				break;
-		} else if (rc == 0) {
-			if (n == 1) {
-				DEBUG2("File_Desc: %d Closed.", fd);
-				return 0;
-			} else
-				break;
-		} else {
-			if (errno == EINTR)
-				goto again;
-
-			DEBUG2("File_Desc: %d \"%s\" (%d)", fd, strerror(errno), errno);
-
-			return -1;
-		}
-	}
-
-	/* Tack a NIL to the end to make is a standard "C" string */
-	*ptr = '\0';
-
-	return (ssize_t)n;
 }
