@@ -1,4 +1,4 @@
-/* $Id: sock.h,v 1.6 2001/10/25 17:02:50 rjkaes Exp $
+/* $Id: sock.h,v 1.10 2002/05/23 18:25:55 rjkaes Exp $
  *
  * See 'sock.c' for a detailed description.
  *
@@ -19,25 +19,17 @@
 #ifndef TINYPROXY_SOCK_H
 #define TINYPROXY_SOCK_H
 
-#include "tinyproxy.h"
-
 #define PEER_IP_LENGTH		16
 #define PEER_STRING_LENGTH	256
 
 #define MAXLINE (1024 * 4)
 
 extern int opensock(char *ip_addr, uint16_t port);
-extern int listen_sock(uint16_t port, socklen_t *addrlen);
+extern int listen_sock(uint16_t port, socklen_t * addrlen);
 
 extern int socket_nonblocking(int sock);
 extern int socket_blocking(int sock);
 
-extern char *getpeer_ip(int fd, char *ipaddr);
-extern char *getpeer_string(int fd, char *string);
-
-extern ssize_t safe_write(int fd, const void *buffer, size_t count);
-extern ssize_t safe_read(int fd, void *buffer, size_t count);
-
-extern ssize_t readline(int fd, char *ptr, size_t maxlen);
+extern int getpeer_information(int fd, char* ipaddr, char* string_addr);
 
 #endif
